/*
 * Copyright 2015 Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.pcepio.types;

import java.util.Objects;

import org.jboss.netty.buffer.ChannelBuffer;
import org.onosproject.pcepio.protocol.PcepVersion;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.base.MoreObjects;

/**
 * Provides Autonomous System Tlv which contains opaque value (32 Bit AS Number).
 */
public class AutonomousSystemTlv implements PcepValueType {

    /* Reference :RFC3209
     *  0                   1                   2                   3
      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
     |           Type=[TBD10]         |             Length=4         |
     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
     |                    opaque value (32 Bit AS Number)            |
     +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
     */

    protected static final Logger log = LoggerFactory.getLogger(AutonomousSystemTlv.class);

    public static final short TYPE = 100; //TODD:change this TBD10
    public static final short LENGTH = 4;

    private final int rawValue;

    /**
     * constructor to initialize Autonomous system tlv value.
     *
     * @param rawValue value of Autonomous system tlv
     */
    public AutonomousSystemTlv(int rawValue) {
        this.rawValue = rawValue;
    }

    /**
     * To create instance of AutonomousSystemTlv.
     *
     * @param raw opaque value ofc 32 Bit AS Number
     * @return object of AutonomousSystemTlv
     */
    public static AutonomousSystemTlv of(final int raw) {
        return new AutonomousSystemTlv(raw);
    }

    /**
     * Returns opaque value of 32 Bit AS Number.
     *
     * @return int value of rawValue
     */
    public int getInt() {
        return rawValue;
    }

    @Override
    public PcepVersion getVersion() {
        return PcepVersion.PCEP_1;
    }

    @Override
    public short getType() {
        return TYPE;
    }

    @Override
    public short getLength() {
        return LENGTH;
    }

    @Override
    public int hashCode() {
        return Objects.hash(rawValue);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj instanceof AutonomousSystemTlv) {
            AutonomousSystemTlv other = (AutonomousSystemTlv) obj;
            return Objects.equals(rawValue, other.rawValue);
        }
        return false;
    }

    @Override
    public int write(ChannelBuffer c) {
        int iLenStartIndex = c.writerIndex();
        c.writeShort(TYPE);
        c.writeShort(LENGTH);
        c.writeInt(rawValue);
        return c.writerIndex() - iLenStartIndex;
    }

    /**
     * Reads the channel buffer and returns object of AutonomousSystemTlv.
     *
     * @param c type of channel buffer
     * @return object of AutonomousSystemTlv
     */
    public static AutonomousSystemTlv read(ChannelBuffer c) {
        return AutonomousSystemTlv.of(c.readInt());
    }

    @Override
    public void print() {
        log.debug("AutonomousSystemTlv");
        log.debug("Type: " + TYPE);
        log.debug("Length: " + LENGTH);
        log.debug("Value: " + rawValue);
    }

    @Override
    public String toString() {
        return MoreObjects.toStringHelper(getClass())
                .add("TYPE", TYPE)
                .add("Length", LENGTH)
                .add("value", rawValue)
                .toString();
    }
}
